﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using DoubleDeckerBus.DrawningObjects;
using DoubleDeckerBus.Entities;
using static System.Windows.Forms.VisualStyles.VisualStyleElement.Button;


namespace DoubleDeckerBus
{
    public partial class FormBusConfig : Form
    {
        DrawningBus? _bus = null;
        private event Action<DrawningBus>? EventAddBus;
      
        public FormBusConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += PanelColor_MouseDown;
            panelPurple.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;
            labelSimpleObject.MouseDown += LabelObject_MouseDown;
            labelModifiedObject.MouseDown += LabelObject_MouseDown;
            buttonCancel.Click += (s, e) => Close();
        }
        public void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor,
                DragDropEffects.Move | DragDropEffects.Copy);
        }
        public void DrawBus()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _bus?.SetPosition(5, 5);
            _bus?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }
        public void AddEvent(Action<DrawningBus>? ev)
        {
            if (EventAddBus == null)
            {
                EventAddBus = ev;
            }
            else
            {
                EventAddBus += ev;
            }
        }
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name, DragDropEffects.Move | DragDropEffects.Copy);
        }
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _bus = new DrawningBus((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, pictureBoxObject.Width,
                   pictureBoxObject.Height);
                    break;
                case "labelModifiedObject":
                    _bus = new DrawningDoubleDeckerBus((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, Color.Black, checkBoxSecondFloor.Checked,
                   checkBoxLadder.Checked, checkBoxLine.Checked, pictureBoxObject.Width,
                   pictureBoxObject.Height);
                    break;
            }
            DrawBus();
        }
        private void LabelColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_bus == null)
                return;
            switch (((Label)sender).Name)
            {
                case "labelColor":
                    _bus?.EntityBus?.setBodyColor((Color)e.Data.GetData(typeof(Color)));
                    break;
                case "labelDopColor":
                    if (!(_bus is DoubleDeckerBus))
                        return;
                    (_bus.EntityBus as EntityDoubleDeckerBus)?.setAdditionalColor(color:
                        (Color)e.Data.GetData(typeof(Color)));
                    break;
            }
            DrawBus();
        }
        private void LabelColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        private void LabelDopColor_DragDrop(object sender, DragEventArgs e)
        {
            if ((_bus?.EntityBus == null) || (_bus is DrawningDoubleDeckerBus == false))
                return;
            Color additionalColor = (Color)e.Data.GetData(typeof(Color));
            _bus = new DrawningDoubleDeckerBus((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, _bus.EntityBus.BodyColor, additionalColor, checkBoxSecondFloor.Checked,
                   checkBoxLadder.Checked, checkBoxLine.Checked, pictureBoxObject.Width,
                   pictureBoxObject.Height);
            DrawBus();
        }
        private void LabelDopColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        private void ButtonOk_Click(object sender, EventArgs e)
        {
            EventAddBus?.Invoke(_bus);
            Close();
        }
    }
}
