﻿using DoubleDeckerBus.Exceptions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Numerics;
using System.Text;
using System.Threading.Tasks;

namespace DoubleDeckerBus.Generics
{
    internal class SetGeneric<T>
    where T : class
    {
        private readonly List<T?> _places;
        public int Count => _places.Count;
        private readonly int _maxCount;
        public SetGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T?>(count);
        }
        public bool Insert(T bus)
        {
            if (_places.Count == _maxCount)
                throw new StorageOverflowException(_maxCount);
            Insert(bus, 0);
            return true;
        }
        public bool Insert(T bus, int position)
        {
            if (_places.Count == _maxCount)
                throw new StorageOverflowException(_maxCount);
            if (!(position >= 0 && position <= Count))
                return false;
            _places.Insert(position, bus);
            return true;
        }
        public bool Remove(int position)
        {
            if (!(position >= 0 && position < Count))
                throw new BusNotFoundException(position);
            _places.RemoveAt(position);
            return true;
        }
        public T? this[int position]
        {
            get
            {
                if (!(position >= 0 && position < Count))
                    return null;
                return _places[position];
            }
            set
            {
                if (!(position >= 0 && position < Count && _places.Count < _maxCount))
                    return;
                _places.Insert(position, value);
                return;
            }
        }
        public IEnumerable<T?> GetTheBuses(int? maxTheBuses = null)
        {
            for (int i = 0; i < _places.Count; ++i)
            {
                yield return _places[i];
                if (maxTheBuses.HasValue && i == maxTheBuses.Value)
                {
                    yield break;
                }
            }
        }
    }
}
